import xbmc
import xbmcvfs
import os
from os.path import join, exists
from urllib2 import urlopen, HTTPError
import json
import datetime
from resources.lib.gui.renderers.dialog_renderer import DialogRenderer
from resources.lib.utils.kodiutils import get_string, notification, show_settings
from resources.lib.storage.settings import settings
from resources.lib.kodilogging import logger
from resources.lib.const import SETTINGS, MEDIA_TYPE, URL, ENDPOINT, ROUTE
from resources.lib.storage.sqlite import DB

def normalizeName(name):
    name = name.replace(': ', ' ')
    name = name.replace(':', ' ')
    name = name.replace('*', '.')
    return name

def getMediaURL(id):
    MEDIA_URL = URL.API + ENDPOINT.MEDIA + '/filter/{filter_name}?{query}'
    return MEDIA_URL.format(filter_name='parent', query='value=' + id + '&sort=episode')

def getMediaDetailURL(id):
    return URL.API + ENDPOINT.MEDIA + "/" + id

def getStreamURL(media, media_id, root_parent_id):
    STREAM_URL = ROUTE.PROCESS_MEDIA_ITEM + "?url=%2Fapi%2Fmedia%2F{media}%2Fstreams&media_id={media_id}&root_parent_id={root_parent_id}" 
    return STREAM_URL.format(media=media, media_id=media_id, root_parent_id=root_parent_id)

def getNewsURL():
    NEWS_URL = URL.API + ENDPOINT.MEDIA + '/filter/{filter_name}?{query}'
    filter_name = 'news'
    query = 'type=movie&sort=dateAdded&order=desc&days=365'
    return NEWS_URL.format(filter_name=filter_name, query=query)
    
def getNewsDubbedURL(lang_main, lang_fallback):
    NEWS_DUBBED_URL = URL.API + ENDPOINT.MEDIA + '/filter/{filter_name}?{query}'
    query = 'type=movie&sort=dateAdded&order=desc&days=365&lang={}'.format(lang_main)
    if(lang_fallback != ''):
        query = query + '&lang=' + lang_fallback
    return NEWS_DUBBED_URL.format(filter_name='newsDubbed', query=query)
    
def writeNFOFile(name, mediaDetail):
    type = 'movie'
    if(mediaDetail.get('info_labels').get('mediatype') == MEDIA_TYPE.TV_SHOW):
        type = 'tv'
    if('services' in mediaDetail):
        services = mediaDetail['services']
        file = xbmcvfs.File(name, 'w')
        if('csfd' in services):
            csfdId = services['csfd'] 
            file.write("https://www.csfd.cz/film/{}".format(csfdId) + "\n")
        if('tmdb' in services):
            tmdbId = services['tmdb']        
            file.write("https://www.themoviedb.org/{}/{}".format(type, tmdbId) + "\n")
        if('imdb' in services):
            imdbId = services['imdb'] 
            file.write("https://www.imdb.com/title/{}".format(imdbId) + "\n")
        file.close()

def writeSTRMFile(path, content):
    file = xbmcvfs.File(path, 'w')
    file.write(content)
    file.close()

def getDeltaFromToday(dateaddedStr):
    today = datetime.date.today()
    addedYear = int(dateaddedStr[:4])
    addedMonth = int(dateaddedStr[5:7])
    addedDay = int(dateaddedStr[8:10])
    dateadded = datetime.date(addedYear, addedMonth, addedDay)
    delta = (today-dateadded).days
    return delta

def add_movie(mf, id, mediaDetail):
    info_labels = mediaDetail.get('info_labels') 
    originalTitle = info_labels.get('originaltitle')
    year = info_labels.get('year')
    if(not originalTitle):
        i18n_info_labels = mediaDetail.get('i18n_info_labels')
        for i18n_info_label in i18n_info_labels:
            if i18n_info_label.get('title'):
                originalTitle = i18n_info_label.get('title')
                break
    
    originalTitle = normalizeName(originalTitle)
    dirName = mf + originalTitle + ' (' + str(year) + ')'
    if not xbmcvfs.exists(dirName):
        xbmcvfs.mkdir(dirName)
    strmFileName = join(dirName, originalTitle + ' (' + str(year) + ')' + '.strm')
    nfoFileName  = join(dirName, originalTitle + ' (' + str(year) + ')' + '.nfo') 

    writeNFOFile(nfoFileName, mediaDetail)
    moviePath = getStreamURL(id, id, id)    
    writeSTRMFile(strmFileName, moviePath)
    

def add_movie_one(mf, mediaid):
    logger.debug("Adding movie to library... BEGIN")
    #year            = xbmc.getInfoLabel("ListItem.Year")
    moviePath       = xbmc.getInfoLabel("ListItem.FileNameAndPath")

    url = getMediaDetailURL(mediaid)
    try:
        detailContents = urlopen(url).read()
    except HTTPError as err:
        logger.error(err)
    
    if(detailContents != ''):
        mediaDetail = json.loads(detailContents)
        add_movie(mf, mediaid, mediaDetail)
        
        logger.debug("Adding movie to library... END")
        xbmc.executebuiltin('UpdateLibrary(video)')

def add_movie_news(mf, id, mediaDetail):
    add_movie(mf, id, mediaDetail)

def add_tvshow(tf, mediaid):
    logger.debug("Adding TV show to library... BEGIN")
    subs = DB.TV_SHOWS_SUBSCRIPTIONS.get_all()
    logger.debug("Subscriptions: {}".format(subs))

    url = getMediaDetailURL(mediaid)
    detailContents = ''
    try:
        detailContents = urlopen(url).read()
    except HTTPError as err:
        logger.error(err)
    if(detailContents != ''):
        dp = DialogRenderer.progress(get_string(30351), get_string(30352))
        mediaDetail = json.loads(detailContents)
        info_labels = mediaDetail.get('info_labels') 
        originalTitle = info_labels.get('originaltitle')
        year = info_labels.get('year')
        if(not originalTitle):
            i18n_info_labels = mediaDetail.get('i18n_info_labels')
            for i18n_info_label in i18n_info_labels:
                if i18n_info_label.get('title'):
                    originalTitle = i18n_info_label.get('title')
                    break
        
        originalTitle = normalizeName(originalTitle)
        dirName = tf + originalTitle + ' (' + str(year) + ')'
        if not xbmcvfs.exists(dirName):
            xbmcvfs.mkdir(dirName)
        strmFileName = join(dirName, originalTitle + ' (' + str(year) + ')' + '.strm')
        nfoFileName  = join(dirName, originalTitle + ' (' + str(year) + ')' + '.nfo') 
    
        writeNFOFile(nfoFileName, mediaDetail)    

        url = getMediaURL(mediaid)
    
        tvShowContents = ''
        try:
            tvShowContents = urlopen(url).read()
        except HTTPError as err:
            logger.error("URL: {}".format(url))
            logger.error(err)
        if(tvShowContents != ''):
            seasonData = json.loads(tvShowContents)
            seasonsCount = seasonData['totalCount']
            mediaid = seasonData['data'][0]['_source']['root_parent']
            
            s = 1;
            seasons = seasonData['data']
            for season in seasons:
                seasonNo = season['_source']['info_labels']['season']
                episodeNo = season['_source']['info_labels']['episode']
                
                if(episodeNo != 0):
                    seasonDirName = join(dirName, "Season 01")
                    if not xbmcvfs.exists(seasonDirName):
                        xbmcvfs.mkdir(seasonDirName)
                    episodeId = season['_id']
                    parentId = season['_source']['root_parent']
                    episodeURL = getStreamURL(episodeId, episodeId, parentId)
                    episodeFileName = "S{}E{}.strm".format(str(seasonNo).zfill(2), str(episodeNo).zfill(2))
                    writeSTRMFile(join(seasonDirName, episodeFileName), str(episodeURL))
                
                else:        
                    currentSeason = "Season {}".format(str(seasonNo).zfill(2))
                    seasonId = season["_id"]
                    seasonDirName = join(dirName, currentSeason)
                    if not xbmcvfs.exists(seasonDirName):
                        xbmcvfs.mkdir(seasonDirName)
                    
                    url = getMediaURL(seasonId)
                    episodesContents = ''
                    try:
                        episodesContents = urlopen(url).read()
                    except HTTPError as err:
                        logger.error("URL: {}".format(url))
                        logger.error(err)
                    if(episodesContents != ''):
                        episodesData = json.loads(episodesContents) 
                        episodes = episodesData['data']
                        episodesCount = episodesData['totalCount']
                        for episode in episodes:
                            seasonNo = episode['_source']['info_labels']['season']
                            episodeNo = episode['_source']['info_labels']['episode']
                            episodeFileName = "S{}E{}.strm".format(str(seasonNo).zfill(2), str(episodeNo).zfill(2))
                            episodeId = episode['_id']
                            episodeURL = getStreamURL(episodeId, episodeId, seasonId)
                            writeSTRMFile(join(seasonDirName, episodeFileName), str(episodeURL))
                s = s+1
                dp.update(int(100*float(s)/float(len(seasons))))
    
        DB.TV_SHOWS_SUBSCRIPTIONS.add(mediaid, dirName)
        subs = DB.TV_SHOWS_SUBSCRIPTIONS.get_all()
        logger.debug("Subscriptions: {}".format(subs))

        dp.close()
        logger.debug("Adding TV show to library... END")
        xbmc.executebuiltin('UpdateLibrary(video)')

def add_to_library(mediaid, mediatype):
    logger.debug("Adding to library, mediaid: {}, type: {}".format(mediaid, mediatype))
    if mediatype == MEDIA_TYPE.MOVIE:
        mf = settings[SETTINGS.MOVIE_LIBRARY_FOLDER]
        add_movie_one(mf, mediaid)

    if (mediatype == MEDIA_TYPE.TV_SHOW):
        tf = settings[SETTINGS.TV_SHOW_LIBRARY_FOLDER]
        add_tvshow(tf, mediaid)

def update_tv_shows():
    run_scan = False
    mf = settings[SETTINGS.MOVIE_LIBRARY_FOLDER]

    if(mf != ''):
        logger.debug("Checking new movies in 'News' and 'News dubbed'... BEGIN")
        
        lang_main = settings.get_select(SETTINGS.PREFERRED_LANGUAGE)
        lang_fallback = settings.get_select(SETTINGS.FALLBACK_LANGUAGE)
        if(lang_fallback is None):
            lang_fallback = ''
        days_options = {0:1, 1:3, 2:5, 3:10}
        days = days_options[settings.as_int(SETTINGS.LIBRARY_AUTO_COUNT)]
        
        # check for new movies
        if settings.as_bool(SETTINGS.LIBRARY_AUTO_NEW):
            url = getNewsURL()
            newsList = ''
            try:
                newsList = urlopen(url).read()
            except HTTPError as err:
                logger.error("URL: {}".format(url))
                logger.error(err)
                
            if(newsList != ''):
                run_scan = True
                result = json.loads(newsList).get('data')
                for movie in result:
                    id = movie.get('_id')
                    delta = getDeltaFromToday(movie.get('_source').get('info_labels').get('dateadded')[:10])
                    logger.debug('Movie id: {}, delta: {}'.format(id, delta))
                    if(delta <= days):                    
                        mediaDetail = movie.get('_source')
                        add_movie_news(mf, id, mediaDetail)
                    else:
                        break

        # check for new movies dubbed in preferred languages
        if settings.as_bool(SETTINGS.LIBRARY_AUTO_NEW_DUBBED):
            url = getNewsDubbedURL(lang_main, lang_fallback)
            newsDubbedList = ''
            try:
                newsDubbedList = urlopen(url).read()
            except HTTPError as err:
                logger.error("URL: {}".format(url))
                logger.error(err)
                
            if(newsDubbedList != ''):
                run_scan = True
                result = json.loads(newsDubbedList).get('data')
                for movie in result:
                    id = movie.get('_id')
                    delta = getDeltaFromToday(movie.get('_source').get('info_labels').get('dateadded')[:10])
                    logger.debug('Movie id: {}, delta: {}'.format(id, delta))
                    if(delta <= days):                    
                        mediaDetail = movie.get('_source')
                        add_movie_news(mf, id, mediaDetail)
                    else:
                        break
    
        logger.debug("Checking new movies in 'News' and 'News dubbed'... END")


    subs = DB.TV_SHOWS_SUBSCRIPTIONS.get_all()
    logger.debug("Subscriptions: {}".format(subs))
    
    list = DB.TV_SHOWS_SUBSCRIPTIONS.get_all()
    if(len(list)) == 0:
        if(run_scan == True):
            xbmc.executebuiltin('UpdateLibrary(video)')
        return
    
    logger.debug("Checking new content for TV shows... BEGIN")

    for item in list:
        mediaid = item[0]
        path = item[1]
        if(not exists(path)):
            # this means TW show has been removed from library, no need subscription
            logger.debug("TV show removed from library, removing subscription {}".format(path))
            DB.TV_SHOWS_SUBSCRIPTIONS.delete(mediaid)
        else:
            logger.debug("Checking new content for {}".format(path))
            # get last season and past episode from users library
            seasons = os.listdir(path)
            lastSeasonStr = ''
            lastSeason = 0
            for item in seasons:
                if(os.path.isdir(os.path.join(path, item))): 
                    lastSeasonStr = item
                    lastSeason = int(item[-2:])

            lastEpisode = 0
            lastEpisodeStr = ''
            episodes = os.listdir(os.path.join(path, lastSeasonStr))
            for item in episodes:
                if(os.path.isfile(os.path.join(path, lastSeasonStr, item)) and (item.endswith('.strm'))):
                    lastEpisodeStr = item
                    lastEpisode = int(item[4:-5]) 
            
            logger.debug('Highest season: {}, highest episode: {}'.format(lastSeason, lastEpisode)) 
            
            # get episodes list by mediaid and add what is missing - only newer episodes (seasons)
            url = getMediaURL(mediaid)
            try:
                tvShowContents = urlopen(url).read()
            except HTTPError as err:
                logger.error("URL: {}".format(url))
                logger.error(err)
            if(tvShowContents != ''):
                seasonData = json.loads(tvShowContents)
                for season in seasonData['data']:
                    seasonNo = season['_source']['info_labels']['season']
                    episodeNo = season['_source']['info_labels']['episode']
                    seasonid = season['_id']
                    root_parent = season['_source']['root_parent']
                    
                    # only one season TV show
                    if(episodeNo != 0):
                        episodeid = season['_id']
                        if(episodeNo > lastEpisode):
                            # missing episode, add it
                            strmPath = join(path, 'Season 01', 'S01E{}.strm'.format(str(episodeNo).zfill(2)))
                            strmContent = getStreamURL(episodeid, episodeid, root_parent)
                            writeSTRMFile(strmPath, strmContent)
                    else:
                        # check highest episode in user library
                        if(seasonNo == lastSeason):
                            url = getMediaURL(seasonid)
                            try:
                                seasonContents = urlopen(url).read()
                            except HTTPError as err:
                                logger.error("URL: {}".format(url))
                                logger.error(err)
                            if(seasonContents != ''):
                                episodesData = json.loads(seasonContents)
                                for episode in episodesData['data']:
                                    episodeNo = episode['_source']['info_labels']['episode'] 
                                    episodeid = episode['_id']
                                    # missing episode, add it
                                    if(episodeNo > lastEpisode):
                                        strmPath = join(path, 'Season {}'.format(str(seasonNo).zfill(2)), 'S{}E{}.strm'.format(str(seasonNo).zfill(2), str(episodeNo).zfill(2)))
                                        strmContent = getStreamURL(episodeid, episodeid, seasonid)
                                        writeSTRMFile(strmPath, strmContent)
                        if(seasonNo > lastSeason):
                            url = getMediaURL(seasonid)
                            try:
                                seasonContents = urlopen(url).read()
                            except HTTPError as err:
                                logger.error("URL: {}".format(url))
                                logger.error(err)
                            if(seasonContents != ''):
                                episodesData = json.loads(seasonContents)
                                # missing season, create season directory and add episodes
                                seasonDirName = join(path, 'Season {}'.format(str(seasonNo).zfill(2)))
                                if not xbmcvfs.exists(seasonDirName):
                                    xbmcvfs.mkdir(seasonDirName)
                                for episode in episodesData['data']:
                                    episodeNo = episode['_source']['info_labels']['episode'] 
                                    episodeid = episode['_id']
                                    strmPath = join(seasonDirName, 'S{}E{}.strm'.format(str(seasonNo).zfill(2), str(episodeNo).zfill(2)))
                                    strmContent = getStreamURL(episodeid, episodeid, seasonid)
                                    writeSTRMFile(strmPath, strmContent)
            
    subs = DB.TV_SHOWS_SUBSCRIPTIONS.get_all()
    logger.debug("Subscriptions: {}".format(subs))
    logger.debug("Checking new content for TV shows... END")
    xbmc.executebuiltin('UpdateLibrary(video)')
